@@ -26,6 +26,8 @@
 #include <color.h>
 #include <glib/gi18n-lib.h>
 #include <glib/gstdio.h>
+#include <float.h>
+#include <math.h>
 #include <gdk/gdk.h>
 
 
@@ -99,6 +101,7 @@ struct _XnpNoteClass {
 };
 
 
+static gpointer xnp_application_parent_class = NULL;
 
 GType xnp_application_get_type (void);
 GType xnp_window_get_type (void);
@@ -111,10 +114,9 @@ static void _g_slist_free_g_object_unref (GSList* self);
 static void xnp_application_set_config_file (XnpApplication* self, const char* value);
 static void _lambda17_ (XfconfChannel* channel, const char* prop, const GValue* val, XnpApplication* self);
 static void __lambda17__xfconf_channel_property_changed (XfconfChannel* _sender, const char* p0, const GValue* p1, gpointer self);
-void xnp_application_create_window (XnpApplication* self, const char* name);
+XnpWindow* xnp_application_create_window (XnpApplication* self, const char* name);
 XnpApplication* xnp_application_new (const char* config_file);
 XnpApplication* xnp_application_construct (GType object_type, const char* config_file);
-XnpApplication* xnp_application_new (const char* config_file);
 XnpWindow* xnp_window_new (void);
 XnpWindow* xnp_window_construct (GType object_type);
 void xnp_window_set_above (XnpWindow* self, gboolean value);
@@ -146,21 +148,25 @@ void xnp_note_set_name (XnpNote* self, const char* value);
 static void _lambda22_ (XnpWindow* win, XnpNote* note, const char* old_name, XnpApplication* self);
 static void __lambda22__xnp_window_note_renamed (XnpWindow* _sender, XnpNote* note, const char* old_name, gpointer self);
 const char* xnp_application_get_config_file (XnpApplication* self);
+void xnp_window_move_note (XnpWindow* self, const char* note_name, gint page);
 void xnp_window_set_current_page (XnpWindow* self, gint page);
 void xnp_window_get_geometry (XnpWindow* self, gint* winx, gint* winy, gint* width, gint* height);
+char** xnp_window_get_note_names (XnpWindow* self, int* result_length1);
 gint xnp_window_get_current_page (XnpWindow* self);
 gboolean xnp_window_get_above (XnpWindow* self);
 gboolean xnp_window_get_sticky (XnpWindow* self);
 void xnp_application_save_windows_configuration (XnpApplication* self);
 void xnp_window_save_notes (XnpWindow* self);
 void xnp_application_save_notes (XnpApplication* self);
+gint xnp_window_get_n_pages (XnpWindow* self);
 void xnp_window_hide (XnpWindow* self);
 void xnp_application_show_hide_notes (XnpApplication* self);
 static GObject * xnp_application_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
-static gpointer xnp_application_parent_class = NULL;
 static void xnp_application_finalize (GObject* obj);
 static void xnp_application_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
 static void xnp_application_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
+static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
+static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
 static int _vala_strcmp0 (const char * str1, const char * str2);
 
 
@@ -212,17 +218,23 @@ XnpApplication* xnp_application_construct (GType object_type, const char* config
 	found = FALSE;
 	{
 		GDir* dir;
-		char* _tmp2_;
-		const char* _tmp1_;
 		dir = g_dir_open (self->priv->notes_path, (guint) 0, &_inner_error_);
 		if (_inner_error_ != NULL) {
 			goto __catch4_g_error;
 			goto __finally4;
 		}
-		_tmp2_ = NULL;
-		_tmp1_ = NULL;
-		while ((name = (_tmp2_ = (_tmp1_ = g_dir_read_name (dir), (_tmp1_ == NULL) ? NULL : g_strdup (_tmp1_)), name = (g_free (name), NULL), _tmp2_)) != NULL) {
-			xnp_application_create_window (self, name);
+		while (TRUE) {
+			char* _tmp2_;
+			const char* _tmp1_;
+			XnpWindow* _tmp3_;
+			_tmp2_ = NULL;
+			_tmp1_ = NULL;
+			if (!((name = (_tmp2_ = (_tmp1_ = g_dir_read_name (dir), (_tmp1_ == NULL) ? NULL : g_strdup (_tmp1_)), name = (g_free (name), NULL), _tmp2_)) != NULL)) {
+				break;
+			}
+			_tmp3_ = NULL;
+			_tmp3_ = xnp_application_create_window (self, name);
+			(_tmp3_ == NULL) ? NULL : (_tmp3_ = (g_object_unref (_tmp3_), NULL));
 			found = TRUE;
 		}
 		(dir == NULL) ? NULL : (dir = (g_dir_close (dir), NULL));
@@ -247,7 +259,10 @@ XnpApplication* xnp_application_construct (GType object_type, const char* config
 		return NULL;
 	}
 	if (found == FALSE) {
-		xnp_application_create_window (self, NULL);
+		XnpWindow* _tmp4_;
+		_tmp4_ = NULL;
+		_tmp4_ = xnp_application_create_window (self, NULL);
+		(_tmp4_ == NULL) ? NULL : (_tmp4_ = (g_object_unref (_tmp4_), NULL));
 	}
 	color = (g_free (color), NULL);
 	name = (g_free (name), NULL);
@@ -275,7 +290,10 @@ static void _lambda18_ (XnpWindow* win, const char* action, XnpApplication* self
 			xnp_application_delete_window (self, win);
 		} else {
 			if (_vala_strcmp0 (action, "create-new-window") == 0) {
-				xnp_application_create_window (self, NULL);
+				XnpWindow* new_win;
+				new_win = xnp_application_create_window (self, NULL);
+				gtk_widget_show ((GtkWidget*) new_win);
+				(new_win == NULL) ? NULL : (new_win = (g_object_unref (new_win), NULL));
 			} else {
 				if (_vala_strcmp0 (action, "properties") == 0) {
 					xnp_application_open_settings_dialog (self);
@@ -393,15 +411,17 @@ static void __lambda22__xnp_window_note_renamed (XnpWindow* _sender, XnpNote* no
  * Creates a new Xnp.Window and stores it inside window_list.
  * If a name is given, it assumes it can load existing notes.
  */
-void xnp_application_create_window (XnpApplication* self, const char* name) {
+XnpWindow* xnp_application_create_window (XnpApplication* self, const char* name) {
+	XnpWindow* result;
 	GError * _inner_error_;
 	XnpWindow* window;
-	XnpWindow* _tmp3_;
-	g_return_if_fail (self != NULL);
+	XnpWindow* _tmp4_;
+	g_return_val_if_fail (self != NULL, NULL);
 	_inner_error_ = NULL;
 	window = g_object_ref_sink (xnp_window_new ());
 	/* Global settings */
 	xfconf_g_property_bind (self->priv->xfconf_channel, "/global/skip-taskbar-hint", G_TYPE_BOOLEAN, window, "skip-taskbar-hint");
+	xfconf_g_property_bind (self->priv->xfconf_channel, "/global/show-tabs", G_TYPE_BOOLEAN, window, "show-tabs");
 	/* Default settings */
 	if (name == NULL) {
 		gint width;
@@ -432,14 +452,25 @@ void xnp_application_create_window (XnpApplication* self, const char* name) {
 		{
 			gint id;
 			id = 1;
-			for (; id <= (len + 1); id++) {
-				if (id > 1) {
-					char* _tmp2_;
-					_tmp2_ = NULL;
-					window_name = (_tmp2_ = g_strdup_printf (_ ("Notes %d"), id), window_name = (g_free (window_name), NULL), _tmp2_);
-				}
-				if (!xnp_application_window_name_exists (self, window_name)) {
-					break;
+			{
+				gboolean _tmp2_;
+				_tmp2_ = TRUE;
+				while (TRUE) {
+					if (!_tmp2_) {
+						id++;
+					}
+					_tmp2_ = FALSE;
+					if (!(id <= (len + 1))) {
+						break;
+					}
+					if (id > 1) {
+						char* _tmp3_;
+						_tmp3_ = NULL;
+						window_name = (_tmp3_ = g_strdup_printf (_ ("Notes %d"), id), window_name = (g_free (window_name), NULL), _tmp3_);
+					}
+					if (!xnp_application_window_name_exists (self, window_name)) {
+						break;
+					}
 				}
 			}
 		}
@@ -449,17 +480,17 @@ void xnp_application_create_window (XnpApplication* self, const char* name) {
 		xnp_window_set_name (window, name);
 	}
 	/* Add to window_list */
-	_tmp3_ = NULL;
-	self->priv->window_list = g_slist_insert_sorted (self->priv->window_list, (_tmp3_ = window, (_tmp3_ == NULL) ? NULL : g_object_ref (_tmp3_)), (GCompareFunc) xnp_window_compare_func);
+	_tmp4_ = NULL;
+	self->priv->window_list = g_slist_insert_sorted (self->priv->window_list, (_tmp4_ = window, (_tmp4_ == NULL) ? NULL : g_object_ref (_tmp4_)), (GCompareFunc) xnp_window_compare_func);
 	{
 		GSList* win_collection;
 		GSList* win_it;
 		win_collection = self->priv->window_list;
 		for (win_it = win_collection; win_it != NULL; win_it = win_it->next) {
-			XnpWindow* _tmp4_;
+			XnpWindow* _tmp5_;
 			XnpWindow* win;
-			_tmp4_ = NULL;
-			win = (_tmp4_ = (XnpWindow*) win_it->data, (_tmp4_ == NULL) ? NULL : g_object_ref (_tmp4_));
+			_tmp5_ = NULL;
+			win = (_tmp5_ = (XnpWindow*) win_it->data, (_tmp5_ == NULL) ? NULL : g_object_ref (_tmp5_));
 			{
 				xnp_window_set_window_list (win, self->priv->window_list);
 				(win == NULL) ? NULL : (win = (g_object_unref (win), NULL));
@@ -504,9 +535,8 @@ void xnp_application_create_window (XnpApplication* self, const char* name) {
 			(window == NULL) ? NULL : (window = (g_object_unref (window), NULL));
 			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
 			g_clear_error (&_inner_error_);
-			return;
+			return NULL;
 		}
-		gtk_widget_show ((GtkWidget*) window);
 		(note == NULL) ? NULL : (note = (g_object_unref (note), NULL));
 		window_path = (g_free (window_path), NULL);
 	} else {
@@ -517,7 +547,8 @@ void xnp_application_create_window (XnpApplication* self, const char* name) {
 	g_signal_connect (window, "note-inserted", (GCallback) __lambda20__xnp_window_note_inserted, self);
 	g_signal_connect (window, "note-deleted", (GCallback) __lambda21__xnp_window_note_deleted, self);
 	g_signal_connect (window, "note-renamed", (GCallback) __lambda22__xnp_window_note_renamed, self);
-	(window == NULL) ? NULL : (window = (g_object_unref (window), NULL));
+	result = window;
+	return result;
 }
 
 
@@ -539,8 +570,6 @@ static void xnp_application_load_window_data (XnpApplication* self, XnpWindow* w
 	path = g_strdup_printf ("%s/%s", self->priv->notes_path, xnp_window_get_name (window));
 	{
 		GDir* dir;
-		char* _tmp1_;
-		const char* _tmp0_;
 		dir = g_dir_open (path, (guint) 0, &_inner_error_);
 		if (_inner_error_ != NULL) {
 			if (_inner_error_->domain == G_FILE_ERROR) {
@@ -548,9 +577,14 @@ static void xnp_application_load_window_data (XnpApplication* self, XnpWindow* w
 			}
 			goto __finally7;
 		}
-		_tmp1_ = NULL;
-		_tmp0_ = NULL;
-		while ((name = (_tmp1_ = (_tmp0_ = g_dir_read_name (dir), (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), name = (g_free (name), NULL), _tmp1_)) != NULL) {
+		while (TRUE) {
+			char* _tmp1_;
+			const char* _tmp0_;
+			_tmp1_ = NULL;
+			_tmp0_ = NULL;
+			if (!((name = (_tmp1_ = (_tmp0_ = g_dir_read_name (dir), (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), name = (g_free (name), NULL), _tmp1_)) != NULL)) {
+				break;
+			}
 			{
 				char* contents;
 				char* filename;
@@ -593,7 +627,7 @@ static void xnp_application_load_window_data (XnpApplication* self, XnpWindow* w
 				e = _inner_error_;
 				_inner_error_ = NULL;
 				{
-					g_warning ("application.vala:212: %s", e->message);
+					g_warning ("application.vala:215: %s", e->message);
 					(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
 				}
 			}
@@ -626,17 +660,22 @@ static void xnp_application_load_window_data (XnpApplication* self, XnpWindow* w
 		g_clear_error (&_inner_error_);
 		return;
 	}
-	/* Load configuration */
+	/* Load window configuration */
 	keyfile = g_key_file_new ();
 	{
 		gint winx;
 		gint winy;
 		gint width;
 		gint height;
+		char** _tmp7_;
+		gint tabs_order_size;
+		gint tabs_order_length1;
+		gint _tmp6_;
+		char** tabs_order;
 		gint last_page;
 		gboolean above;
 		gboolean sticky;
-		gint _tmp6_;
+		gint _tmp8_;
 		double opacity;
 		gboolean visible;
 		g_key_file_load_from_file (keyfile, self->priv->_config_file, G_KEY_FILE_NONE, &_inner_error_);
@@ -664,34 +703,63 @@ static void xnp_application_load_window_data (XnpApplication* self, XnpWindow* w
 			goto __catch9_g_error;
 			goto __finally9;
 		}
+		_tmp7_ = NULL;
+		tabs_order = (_tmp7_ = g_key_file_get_string_list (keyfile, xnp_window_get_name (window), "TabsOrder", &_tmp6_, &_inner_error_), tabs_order_length1 = _tmp6_, tabs_order_size = tabs_order_length1, _tmp7_);
+		if (_inner_error_ != NULL) {
+			goto __catch9_g_error;
+			goto __finally9;
+		}
 		last_page = g_key_file_get_integer (keyfile, xnp_window_get_name (window), "LastTab", &_inner_error_);
 		if (_inner_error_ != NULL) {
+			tabs_order = (_vala_array_free (tabs_order, tabs_order_length1, (GDestroyNotify) g_free), NULL);
 			goto __catch9_g_error;
 			goto __finally9;
 		}
 		above = g_key_file_get_boolean (keyfile, xnp_window_get_name (window), "Above", &_inner_error_);
 		if (_inner_error_ != NULL) {
+			tabs_order = (_vala_array_free (tabs_order, tabs_order_length1, (GDestroyNotify) g_free), NULL);
 			goto __catch9_g_error;
 			goto __finally9;
 		}
 		sticky = g_key_file_get_boolean (keyfile, xnp_window_get_name (window), "Sticky", &_inner_error_);
 		if (_inner_error_ != NULL) {
+			tabs_order = (_vala_array_free (tabs_order, tabs_order_length1, (GDestroyNotify) g_free), NULL);
 			goto __catch9_g_error;
 			goto __finally9;
 		}
-		_tmp6_ = g_key_file_get_integer (keyfile, xnp_window_get_name (window), "Transparency", &_inner_error_);
+		_tmp8_ = g_key_file_get_integer (keyfile, xnp_window_get_name (window), "Transparency", &_inner_error_);
 		if (_inner_error_ != NULL) {
+			tabs_order = (_vala_array_free (tabs_order, tabs_order_length1, (GDestroyNotify) g_free), NULL);
 			goto __catch9_g_error;
 			goto __finally9;
 		}
-		opacity = 1 - (((double) _tmp6_) / 100);
+		opacity = 1 - (((double) _tmp8_) / 100);
 		visible = g_key_file_get_boolean (keyfile, xnp_window_get_name (window), "Visible", &_inner_error_);
 		if (_inner_error_ != NULL) {
+			tabs_order = (_vala_array_free (tabs_order, tabs_order_length1, (GDestroyNotify) g_free), NULL);
 			goto __catch9_g_error;
 			goto __finally9;
 		}
 		gtk_window_move ((GtkWindow*) window, winx, winy);
 		gtk_window_resize ((GtkWindow*) window, width, height);
+		{
+			gint i;
+			i = 0;
+			{
+				gboolean _tmp9_;
+				_tmp9_ = TRUE;
+				while (TRUE) {
+					if (!_tmp9_) {
+						i++;
+					}
+					_tmp9_ = FALSE;
+					if (!(i < tabs_order_length1)) {
+						break;
+					}
+					xnp_window_move_note (window, tabs_order[i], i);
+				}
+			}
+		}
 		xnp_window_set_current_page (window, last_page);
 		xnp_window_set_above (window, above);
 		xnp_window_set_sticky (window, sticky);
@@ -699,6 +767,7 @@ static void xnp_application_load_window_data (XnpApplication* self, XnpWindow* w
 		if (visible) {
 			gtk_widget_show ((GtkWidget*) window);
 		}
+		tabs_order = (_vala_array_free (tabs_order, tabs_order_length1, (GDestroyNotify) g_free), NULL);
 	}
 	goto __finally9;
 	__catch9_g_error:
@@ -707,7 +776,7 @@ static void xnp_application_load_window_data (XnpApplication* self, XnpWindow* w
 		e = _inner_error_;
 		_inner_error_ = NULL;
 		{
-			g_warning ("application.vala:243: %s: %s", self->priv->_config_file, e->message);
+			g_warning ("application.vala:249: Unable to load window configuration from %s: %s", self->priv->_config_file, e->message);
 			gtk_widget_show ((GtkWidget*) window);
 			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
 		}
@@ -745,15 +814,20 @@ void xnp_application_save_windows_configuration (XnpApplication* self) {
 			GSList* win_it;
 			win_collection = self->priv->window_list;
 			for (win_it = win_collection; win_it != NULL; win_it = win_it->next) {
-				XnpWindow* _tmp0_;
+				XnpWindow* _tmp2_;
 				XnpWindow* win;
-				_tmp0_ = NULL;
-				win = (_tmp0_ = (XnpWindow*) win_it->data, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
+				_tmp2_ = NULL;
+				win = (_tmp2_ = (XnpWindow*) win_it->data, (_tmp2_ == NULL) ? NULL : g_object_ref (_tmp2_));
 				{
 					gint winx;
 					gint winy;
 					gint width;
 					gint height;
+					char** _tmp1_;
+					gint tabs_order_size;
+					gint tabs_order_length1;
+					gint _tmp0_;
+					char** tabs_order;
 					gint last_page;
 					gint transparency;
 					gboolean visible;
@@ -762,6 +836,8 @@ void xnp_application_save_windows_configuration (XnpApplication* self) {
 					width = 0;
 					height = 0;
 					xnp_window_get_geometry (win, &winx, &winy, &width, &height);
+					_tmp1_ = NULL;
+					tabs_order = (_tmp1_ = xnp_window_get_note_names (win, &_tmp0_), tabs_order_length1 = _tmp0_, tabs_order_size = tabs_order_length1, _tmp1_);
 					last_page = xnp_window_get_current_page (win);
 					transparency = (gint) ((1 - gtk_window_get_opacity ((GtkWindow*) win)) * 100);
 					visible = (gboolean) (GTK_WIDGET_FLAGS ((GtkWidget*) win) & GTK_VISIBLE);
@@ -769,12 +845,14 @@ void xnp_application_save_windows_configuration (XnpApplication* self) {
 					g_key_file_set_integer (keyfile, xnp_window_get_name (win), "PosY", winy);
 					g_key_file_set_integer (keyfile, xnp_window_get_name (win), "Width", width);
 					g_key_file_set_integer (keyfile, xnp_window_get_name (win), "Height", height);
+					g_key_file_set_string_list (keyfile, xnp_window_get_name (win), "TabsOrder", tabs_order, tabs_order_length1);
 					g_key_file_set_integer (keyfile, xnp_window_get_name (win), "LastTab", last_page);
 					g_key_file_set_boolean (keyfile, xnp_window_get_name (win), "Above", xnp_window_get_above (win));
 					g_key_file_set_boolean (keyfile, xnp_window_get_name (win), "Sticky", xnp_window_get_sticky (win));
 					g_key_file_set_double (keyfile, xnp_window_get_name (win), "Transparency", (double) transparency);
 					g_key_file_set_boolean (keyfile, xnp_window_get_name (win), "Visible", visible);
 					(win == NULL) ? NULL : (win = (g_object_unref (win), NULL));
+					tabs_order = (_vala_array_free (tabs_order, tabs_order_length1, (GDestroyNotify) g_free), NULL);
 				}
 			}
 		}
@@ -879,7 +957,7 @@ static void xnp_application_save_note (XnpApplication* self, XnpWindow* window,
 		e = _inner_error_;
 		_inner_error_ = NULL;
 		{
-			g_warning ("application.vala:306: %s", e->message);
+			g_warning ("application.vala:314: %s", e->message);
 			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
 		}
 	}
@@ -957,26 +1035,27 @@ static void xnp_application_rename_window (XnpApplication* self, XnpWindow* wind
  */
 static void xnp_application_delete_window (XnpApplication* self, XnpWindow* window) {
 	GError * _inner_error_;
-	GtkMessageDialog* dialog;
-	gint res;
 	char* name;
 	char* path;
 	g_return_if_fail (self != NULL);
 	g_return_if_fail (window != NULL);
 	_inner_error_ = NULL;
-	dialog = g_object_ref_sink ((GtkMessageDialog*) gtk_message_dialog_new ((GtkWindow*) window, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO, _ ("Are you sure you want to delete this group?"), NULL));
-	res = gtk_dialog_run ((GtkDialog*) dialog);
-	gtk_object_destroy ((GtkObject*) dialog);
-	if (res != GTK_RESPONSE_YES) {
+	if (xnp_window_get_n_pages (window) >= 1) {
+		GtkMessageDialog* dialog;
+		gint res;
+		dialog = g_object_ref_sink ((GtkMessageDialog*) gtk_message_dialog_new ((GtkWindow*) window, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO, _ ("Are you sure you want to delete this group?"), NULL));
+		res = gtk_dialog_run ((GtkDialog*) dialog);
+		gtk_object_destroy ((GtkObject*) dialog);
+		if (res != GTK_RESPONSE_YES) {
+			(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
+			return;
+		}
 		(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
-		return;
 	}
 	name = NULL;
 	path = g_strdup_printf ("%s/%s", self->priv->notes_path, xnp_window_get_name (window));
 	{
 		GDir* dir;
-		char* _tmp1_;
-		const char* _tmp0_;
 		dir = g_dir_open (path, (guint) 0, &_inner_error_);
 		if (_inner_error_ != NULL) {
 			if (_inner_error_->domain == G_FILE_ERROR) {
@@ -984,10 +1063,15 @@ static void xnp_application_delete_window (XnpApplication* self, XnpWindow* wind
 			}
 			goto __finally12;
 		}
-		_tmp1_ = NULL;
-		_tmp0_ = NULL;
-		while ((name = (_tmp1_ = (_tmp0_ = g_dir_read_name (dir), (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), name = (g_free (name), NULL), _tmp1_)) != NULL) {
+		while (TRUE) {
+			char* _tmp1_;
+			const char* _tmp0_;
 			char* filename;
+			_tmp1_ = NULL;
+			_tmp0_ = NULL;
+			if (!((name = (_tmp1_ = (_tmp0_ = g_dir_read_name (dir), (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), name = (g_free (name), NULL), _tmp1_)) != NULL)) {
+				break;
+			}
 			filename = g_strdup_printf ("%s/%s", path, name);
 			g_unlink (filename);
 			filename = (g_free (filename), NULL);
@@ -1007,7 +1091,6 @@ static void xnp_application_delete_window (XnpApplication* self, XnpWindow* wind
 	}
 	__finally12:
 	if (_inner_error_ != NULL) {
-		(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
 		name = (g_free (name), NULL);
 		path = (g_free (path), NULL);
 		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
@@ -1033,9 +1116,11 @@ static void xnp_application_delete_window (XnpApplication* self, XnpWindow* wind
 			}
 		}
 	} else {
-		xnp_application_create_window (self, NULL);
+		XnpWindow* new_win;
+		new_win = xnp_application_create_window (self, NULL);
+		gtk_widget_show ((GtkWidget*) new_win);
+		(new_win == NULL) ? NULL : (new_win = (g_object_unref (new_win), NULL));
 	}
-	(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
 	name = (g_free (name), NULL);
 	path = (g_free (path), NULL);
 }
@@ -1047,6 +1132,7 @@ static void xnp_application_delete_window (XnpApplication* self, XnpWindow* wind
  * Verify if the given name already exists in the window list.
  */
 static gboolean xnp_application_window_name_exists (XnpApplication* self, const char* name) {
+	gboolean result;
 	g_return_val_if_fail (self != NULL, FALSE);
 	g_return_val_if_fail (name != NULL, FALSE);
 	{
@@ -1054,20 +1140,22 @@ static gboolean xnp_application_window_name_exists (XnpApplication* self, const
 		GSList* win_it;
 		win_collection = self->priv->window_list;
 		for (win_it = win_collection; win_it != NULL; win_it = win_it->next) {
-			XnpWindow* _tmp1_;
+			XnpWindow* _tmp0_;
 			XnpWindow* win;
-			_tmp1_ = NULL;
-			win = (_tmp1_ = (XnpWindow*) win_it->data, (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_));
+			_tmp0_ = NULL;
+			win = (_tmp0_ = (XnpWindow*) win_it->data, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
 			{
 				if (_vala_strcmp0 (xnp_window_get_name (win), name) == 0) {
-					gboolean _tmp0_;
-					return (_tmp0_ = TRUE, (win == NULL) ? NULL : (win = (g_object_unref (win), NULL)), _tmp0_);
+					result = TRUE;
+					(win == NULL) ? NULL : (win = (g_object_unref (win), NULL));
+					return result;
 				}
 				(win == NULL) ? NULL : (win = (g_object_unref (win), NULL));
 			}
 		}
 	}
-	return FALSE;
+	result = FALSE;
+	return result;
 }
 
 
@@ -1077,6 +1165,7 @@ static gboolean xnp_application_window_name_exists (XnpApplication* self, const
  * Verify if the given name is valid for window and notes.
  */
 static gboolean xnp_application_name_is_valid (XnpApplication* self, const char* name) {
+	gboolean result;
 	gboolean res;
 	g_return_val_if_fail (self != NULL, FALSE);
 	g_return_val_if_fail (name != NULL, FALSE);
@@ -1092,7 +1181,8 @@ static gboolean xnp_application_name_is_valid (XnpApplication* self, const char*
 		gtk_object_destroy ((GtkObject*) error_dialog);
 		(error_dialog == NULL) ? NULL : (error_dialog = (g_object_unref (error_dialog), NULL));
 	}
-	return res;
+	result = res;
+	return result;
 }
 
 
@@ -1211,18 +1301,20 @@ void xnp_application_open_settings_dialog (XnpApplication* self) {
 
 
 const char* xnp_application_get_config_file (XnpApplication* self) {
+	const char* result;
 	g_return_val_if_fail (self != NULL, NULL);
-	return self->priv->_config_file;
+	result = self->priv->_config_file;
+	return result;
 }
 
 
 static void xnp_application_set_config_file (XnpApplication* self, const char* value) {
-	char* _tmp2_;
-	const char* _tmp1_;
+	char* _tmp1_;
+	const char* _tmp0_;
 	g_return_if_fail (self != NULL);
-	_tmp2_ = NULL;
 	_tmp1_ = NULL;
-	self->priv->_config_file = (_tmp2_ = (_tmp1_ = value, (_tmp1_ == NULL) ? NULL : g_strdup (_tmp1_)), self->priv->_config_file = (g_free (self->priv->_config_file), NULL), _tmp2_);
+	_tmp0_ = NULL;
+	self->priv->_config_file = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_config_file = (g_free (self->priv->_config_file), NULL), _tmp1_);
 	g_object_notify ((GObject *) self, "config-file");
 }
 
@@ -1316,6 +1408,24 @@ static void xnp_application_set_property (GObject * object, guint property_id, c
 }
 
 
+static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
+	if ((array != NULL) && (destroy_func != NULL)) {
+		int i;
+		for (i = 0; i < array_length; i = i + 1) {
+			if (((gpointer*) array)[i] != NULL) {
+				destroy_func (((gpointer*) array)[i]);
+			}
+		}
+	}
+}
+
+
+static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
+	_vala_array_destroy (array, array_length, destroy_func);
+	g_free (array);
+}
+
+
 static int _vala_strcmp0 (const char * str1, const char * str2) {
 	if (str1 == NULL) {
 		return -(str1 != str2);
